
use Test::More tests => 1;
BEGIN { use_ok('Math::Int128') };

#
# I would really appreciate somebody contributing a set of tests for
# this module!
#
#                                                 - Salva
